@@ -17,30 +17,22 @@
 #pragma once
 
 #include <thrust/detail/config.h>
+
 #include <thrust/detail/execution_policy.h>
 
 THRUST_NAMESPACE_BEGIN
 namespace detail
 {
 
-template<typename System, typename Allocator, typename InputIterator, typename Pointer>
-THRUST_HOST_DEVICE
-  Pointer copy_construct_range(thrust::execution_policy<System> &from_system,
-                               Allocator &a,
-                               InputIterator first,
-                               InputIterator last,
-                               Pointer result);
+template <typename System, typename Allocator, typename InputIterator, typename Pointer>
+THRUST_HOST_DEVICE Pointer copy_construct_range(
+  thrust::execution_policy<System>& from_system, Allocator& a, InputIterator first, InputIterator last, Pointer result);
 
-template<typename System, typename Allocator, typename InputIterator, typename Size, typename Pointer>
-THRUST_HOST_DEVICE
-  Pointer copy_construct_range_n(thrust::execution_policy<System> &from_system,
-                                 Allocator &a,
-                                 InputIterator first,
-                                 Size n,
-                                 Pointer result);
+template <typename System, typename Allocator, typename InputIterator, typename Size, typename Pointer>
+THRUST_HOST_DEVICE Pointer copy_construct_range_n(
+  thrust::execution_policy<System>& from_system, Allocator& a, InputIterator first, Size n, Pointer result);
 
-} // end detail
+} // namespace detail
 THRUST_NAMESPACE_END
 
 #include <thrust/detail/allocator/copy_construct_range.inl>
-
