@@ -18,8 +18,8 @@
 
 #include <thrust/detail/config.h>
 
-#include <thrust/detail/type_traits/has_nested_type.h>
 #include <thrust/detail/type_traits.h>
+#include <thrust/detail/type_traits/has_nested_type.h>
 
 THRUST_NAMESPACE_BEGIN
 
@@ -28,15 +28,10 @@ namespace detail
 
 __THRUST_DEFINE_HAS_NESTED_TYPE(is_metafunction_defined, type)
 
-template<typename Metafunction>
-  struct enable_if_defined
-    : thrust::detail::lazy_enable_if<
-        is_metafunction_defined<Metafunction>::value,
-        Metafunction
-      >
+template <typename Metafunction>
+struct enable_if_defined : thrust::detail::lazy_enable_if<is_metafunction_defined<Metafunction>::value, Metafunction>
 {};
 
-} // end detail
+} // namespace detail
 
 THRUST_NAMESPACE_END
-
