@@ -14,10 +14,10 @@
  *  limitations under the License.
  */
 
-
 #pragma once
 
 #include <thrust/detail/config.h>
+
 #include <thrust/system/detail/generic/tag.h>
 
 THRUST_NAMESPACE_BEGIN
@@ -28,16 +28,9 @@ namespace detail
 namespace generic
 {
 
-
-template<typename DerivedPolicy,
-         typename ForwardIterator,
-         typename UnaryOperation>
-THRUST_HOST_DEVICE
-  void tabulate(thrust::execution_policy<DerivedPolicy> &exec,
-                ForwardIterator first,
-                ForwardIterator last,
-                UnaryOperation unary_op);
-
+template <typename DerivedPolicy, typename ForwardIterator, typename UnaryOperation>
+THRUST_HOST_DEVICE void tabulate(
+  thrust::execution_policy<DerivedPolicy>& exec, ForwardIterator first, ForwardIterator last, UnaryOperation unary_op);
 
 } // end namespace generic
 } // end namespace detail
@@ -45,4 +38,3 @@ THRUST_HOST_DEVICE
 THRUST_NAMESPACE_END
 
 #include <thrust/system/detail/generic/tabulate.inl>
-
