@@ -14,10 +14,10 @@
  *  limitations under the License.
  */
 
-
 #pragma once
 
 #include <thrust/detail/config.h>
+
 #include <thrust/system/detail/generic/tag.h>
 
 THRUST_NAMESPACE_BEGIN
@@ -28,24 +28,13 @@ namespace detail
 namespace generic
 {
 
-template<typename DerivedPolicy,
-         typename ForwardIterator,
-         typename T>
-THRUST_HOST_DEVICE
-  void uninitialized_fill(thrust::execution_policy<DerivedPolicy> &exec,
-                          ForwardIterator first,
-                          ForwardIterator last,
-                          const T &x);
+template <typename DerivedPolicy, typename ForwardIterator, typename T>
+THRUST_HOST_DEVICE void uninitialized_fill(
+  thrust::execution_policy<DerivedPolicy>& exec, ForwardIterator first, ForwardIterator last, const T& x);
 
-template<typename DerivedPolicy,
-         typename ForwardIterator,
-         typename Size,
-         typename T>
-THRUST_HOST_DEVICE
-  ForwardIterator uninitialized_fill_n(thrust::execution_policy<DerivedPolicy> &exec,
-                                       ForwardIterator first,
-                                       Size n,
-                                       const T &x);
+template <typename DerivedPolicy, typename ForwardIterator, typename Size, typename T>
+THRUST_HOST_DEVICE ForwardIterator
+uninitialized_fill_n(thrust::execution_policy<DerivedPolicy>& exec, ForwardIterator first, Size n, const T& x);
 
 } // end namespace generic
 } // end namespace detail
@@ -53,4 +42,3 @@ THRUST_HOST_DEVICE
 THRUST_NAMESPACE_END
 
 #include <thrust/system/detail/generic/uninitialized_fill.inl>
-
