@@ -26,14 +26,10 @@ namespace detail
 namespace sequential
 {
 
-
 THRUST_EXEC_CHECK_DISABLE
-template<typename BidirectionalIterator1,
-         typename BidirectionalIterator2>
-THRUST_HOST_DEVICE
-BidirectionalIterator2 copy_backward(BidirectionalIterator1 first, 
-                                     BidirectionalIterator1 last, 
-                                     BidirectionalIterator2 result)
+template <typename BidirectionalIterator1, typename BidirectionalIterator2>
+THRUST_HOST_DEVICE BidirectionalIterator2
+copy_backward(BidirectionalIterator1 first, BidirectionalIterator1 last, BidirectionalIterator2 result)
 {
   while (first != last)
   {
@@ -45,9 +41,7 @@ BidirectionalIterator2 copy_backward(BidirectionalIterator1 first,
   return result;
 }
 
-
 } // end namespace sequential
 } // end namespace detail
 } // end namespace system
 THRUST_NAMESPACE_END
-
