@@ -14,14 +14,14 @@
  *  limitations under the License.
  */
 
-
 /*! \file find.h
- *  \brief OpenMP implementation of find_if. 
+ *  \brief OpenMP implementation of find_if.
  */
 
 #pragma once
 
 #include <thrust/detail/config.h>
+
 #include <thrust/system/detail/generic/find.h>
 #include <thrust/system/omp/detail/execution_policy.h>
 
@@ -34,10 +34,7 @@ namespace detail
 {
 
 template <typename DerivedPolicy, typename InputIterator, typename Predicate>
-InputIterator find_if(execution_policy<DerivedPolicy> &exec,
-                      InputIterator first,
-                      InputIterator last,
-                      Predicate pred)
+InputIterator find_if(execution_policy<DerivedPolicy>& exec, InputIterator first, InputIterator last, Predicate pred)
 {
   // omp prefers generic::find_if to cpp::find_if
   return thrust::system::detail::generic::find_if(exec, first, last, pred);
@@ -47,4 +44,3 @@ InputIterator find_if(execution_policy<DerivedPolicy> &exec,
 } // end namespace omp
 } // end namespace system
 THRUST_NAMESPACE_END
-
